/* ------------------------------------------------------------------
 * Un exemple d'implmentation de malloc()/free()
 * Jean-Paul Rigault
 * ------------------------------------------------------------------
 * Programme de test
 * ------------------------------------------------------------------
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include "mymalloc.h"

#define N 100		/* nombre d'allocations par dfaut */
#define SZMAX 10000	/* taille maximum  allouer par dfaut */

/* On effectue nloop allocations de taille tire au hasard (mais infrieure 
 * szmax), puis on dsalloue dans l'ordre inverse.
 */

int main(int argc, char *argv[])
{
    int i; 		/* indice de boucle d'allocation */
    int sz; 		/* taille  allouer (en caractres) */
    void *tab[N];	/* rcupration des adresses de zone alloues */
    int nloop = N;	/* nombre d'allocations */
    int szmax = SZMAX;	/* taille maximale d'allocation */

    /* Arguments de la ligne de commande */
    switch (argc)
    {
    case 1:
        break;
    case 2:
        nloop = atoi(argv[1]);
        if (nloop > N) nloop = N;
        break;
    case 3:
        nloop = atoi(argv[1]);
        if (nloop > N) nloop = N;
        szmax = atoi(argv[2]);
        break;
    default:
        fprintf(stderr,"Usage: tst_mymalloc1 [nloop [szmax]]\n");
        exit(0);
    }

    printf("Starting mymalloc1 test: nloop = %d szmax = %d\n", nloop, szmax);
    mymalloc_instrum();
  
    /* Boucle d'allocation */
    for (i = 0; i < nloop;  ++i)
    {
        sz = rand() % szmax;	/* on tire au sort la taille (entre 0 et
                                 * SZMAX-1) */
        printf("Trying to Allocate size %d at tab[%d]\n", sz, i);
        tab[i] = mymalloc(sz);
        if (tab[i] != NULL)
            printf("Allocated size %d at tab[%d] = 0x%x\n",
                   sz, i, (unsigned)tab[i]);
        else 
            printf("Failed to allocate size %d at tab[%d]\n", sz, i);
        mymalloc_instrum();
    }

    /* Boucle de dallocation, en sens inverse */
    for (i = nloop - 1; i >= 0; --i)
    {
        printf("Trying to deallocate tab[%d]\n", i);
        if (tab[i] != NULL) myfree(tab[i]);
        printf("Dellocated tab[%d] = 0x%x\n", i, (unsigned)tab[i]);
        mymalloc_instrum();
    }

    printf("End of mymalloc1 test\n");
    mymalloc_instrum();
    return 0;
  
}
