/* ------------------------------------------------------------------
 * Un exemple d'implmentation de malloc()/free()
 * Jean-Paul Rigault
 * ------------------------------------------------------------------
 * Programme de test
 * ------------------------------------------------------------------
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include "mymalloc.h"

#define N 1000		/* nombre d'allocations par dfaut et nombre maximal */
#define SZMAX 10000	/* taille maximum  allouer par dfaut */

/* On effectue nloop itrations. A chaque itration on tire  pile ou face pour
 * savoir si l'on fait une allocation ou une dsallocation.
 *
 * Ce test tente donc de mlanger alatoirement allocations et dallocations.
 */

int main(int argc, char *argv[])
{
    int i; 		/* indice de boucle */
    int n;		/* nombre d'allocations */
    int sz; 		/* taille  allouer (en caractres) */
    void *tab[N];	/* rcupration des adresses de zone alloues */
    int nloop = N;	/* nombre d'itrations */
    int szmax = SZMAX;	/* taille maximale d'allocation */
    int nalloc = 0;	/* nombre total d'allocations */
    int ndealloc = 0;	/* nombre total dde dsallocations */

    /* Arguments de la ligne de commande */
    switch (argc)
    {
    case 1:
        break;
    case 2:
        nloop = atoi(argv[1]);
        if (nloop > N) nloop = N;
        break;
    case 3:
        nloop = atoi(argv[1]);
        if (nloop > N) nloop = N;
        szmax = atoi(argv[2]);
        break;
    default:
        fprintf(stderr,"Usage: tst_mymalloc2 [nloop [szmax]]\n");
        exit(0);
    }

    printf("Starting mymalloc2 test: nloop = %d szmax = %d\n", nloop, szmax);
    mymalloc_instrum();
   
    /* Boucle d'allocation */
    for (i = 0, n = 0; i < nloop;  ++i)
    {
        /* Ce n'est pas vraiment pile ou face ! en moyenne 2 fois moins de
         * dsallocations que d'allocations 
         */
        if (rand() % 3 == 0 && n > 0)
        {
            /* On dsalloue le dernier */
            if (tab[--n] != 0) myfree(tab[n]);
            printf("Dellocated tab[%d] = 0x%x\n", n, (unsigned)tab[n]);
            mymalloc_instrum();
            ++ndealloc;
        }
        else
        {
            /* On alloue */
            sz = rand() % szmax;	/* on tire au sort la taille (entre 0 et
                                         * SZMAX-1) */
            tab[n] = mymalloc(sz);
            if (tab[n] != NULL)
            {
                printf("Allocated size %d at tab[%d] = 0x%x\n",
                       sz, n, (unsigned)tab[n]);
                ++nalloc;
                ++n;
            }
            else 
                printf("Failed to allocate size %d at tab[%d]\n", sz, n);
            mymalloc_instrum(); 
        }
    }

    /* Boucle de dallocation de ce qui reste, en sens inverse */
    for (i = n - 1; i >= 0; --i)
    {
        printf("Trying to deallocate tab[%d]\n", i);
        if (tab[i] != NULL) myfree(tab[i]);
        mymalloc_instrum();
        printf("Dellocated tab[%d] = 0x%x\n", i, (unsigned)tab[i]);
    }

    printf("End of mymalloc2 test\n");
    printf("nalloc = %d ndealloc = %d\n", nalloc, ndealloc);
    mymalloc_instrum();
    return 0;
  }
