/* ------------------------------------------------------------------
 * Un exemple d'implmentation de malloc()/free()
 * Jean-Paul Rigault
 * ------------------------------------------------------------------
 * Configuration de l'implmentation
 * ------------------------------------------------------------------
 */

#ifndef _MYMALLOC_IMPL_H_
#define _MYMALLOC_IMPL_H_

/* L'entte d'un bloc
 * ------------------
 *
 * NOTE : dans cette version simple, on ne tient pas compte
 * des contraintes d'alignement des adresses mmoire. De toutes faons la
 * structure suivante devrait tre bien aligne sur la plupart des machines
 */
struct Header
{
    unsigned int sizenh;	/* taille totale (utile + header de gestion) du
                                 * bloc en nombre d'entte */
    struct Header *next;	/* Pointeur sur le bloc suivant dans la liste libre */
};

/* La taille de l'entte
 * ---------------------
 *
 * C'est l'unit d'allocation de l'espace mmoire. Toutes les autres tailles
 * sont exprimes en multiple de HEADER_SZ et sont nommes avec le sufffixe
 * NH (nombre de "headers")
 */
#define HEADER_SZ sizeof(struct Header) /* taille en caractres */

/* La taille minimale (en nombre de headers) de mmoire demande au systme
 * ------------------------------------------------------------------------
 *
 * Lorsqu'on rclame de la mmoire au systme grce  l'appel de sbrk(), on
 * demande au moins un bloc de cette taille (en fait un multiple
 * MINSYSTBLK_NH*HEADER_SZ)
 *
 * NOTE : en gnral cette taille est un multiple de la taille de page du
 * systme. Nous ne tenons pas compte de cette contrainte ici.
 */
#define MINSYSTBLOCK_NH 100

/* Fonction d'arrondi
 * ------------------
 *
 * NBLOCKS(sz, blksz) cherche combien de blocks de taille blksz sont
 * ncessaires au minimum pour contenir une information de taille sz. Bien
 * entendu, sz et blksz doivent tre dans la mme unit
 */
#define NBLOCKS(sz, blocksz) (1 + ((sz) - 1) / (blocksz))

/* Instrumentation
 * ---------------
 */
static int nsysblk = 0;		/* nombre de blocs demands  sbrk */
static int syssz = 0;		/* taille totale demande  sbrk */
static int nuserblk = 0;	/* nombre de blocs allous pour l'utilisateur */
static int totalnh = 0;		/* nombre total de blocs entte allous */
static int usersz = 0;		/* taille totale en caractres allloue pour
                                 * l'utilisateur */
static int nfree;		/* nombre de blocs dans la liste libre */
#endif

